# -*- encoding: utf-8 -*-
"""
Serialize data to/from JSON
"""

from StringIO import StringIO

from django.conf import settings
from django.utils import simplejson
from django.core.serializers.json import DjangoJSONEncoder

from python import Serializer as PythonSerializer, Deserializer as PythonDeserializer


class Serializer(PythonSerializer):
    """
    Convert a queryset to JSON.
    """
    def end_serialization(self):
        """Output a JSON encoded queryset."""
        self.options.pop('stream', None)
        self.options.pop('fields', None)
        self.options.pop('excludes', None)
        self.options.pop('relations', None)
        self.options.pop('extras', None)
        self.options.pop('field_extras', None)
        DjangoJSONEncoder.DATE_FORMAT = settings.JSON_DATE_FORMAT
        DjangoJSONEncoder.TIME_FORMAT = settings.JSON_TIME_FORMAT

        simplejson.dump(self.objects, self.stream, cls=DjangoJSONEncoder,
            **self.options)

    def getvalue(self):
        """
        Return the fully serialized queryset (or None if the output stream
        is not seekable).
        """

        if callable(getattr(self.stream, 'getvalue', None)):
            return self.stream.getvalue()


def Deserializer(stream_or_string, **options):
    """ Deserialize a stream or string of JSON data. """
    if isinstance(stream_or_string, basestring):
        stream = StringIO(stream_or_string)
    else:
        stream = stream_or_string
    for obj in PythonDeserializer(simplejson.load(stream), **options):
        yield obj

        

