# -*- encoding: utf-8 -*-

from django.db.models import Q
from django import forms
from django.forms.models import inlineformset_factory

from timetable.models import Gym, TTItem, TTFix, Period


class TTItemForm(forms.ModelForm):
    class Meta:
        model = TTItem
        fields = ['start', 'end']

    start = forms.TimeField(required = False, widget = forms.TextInput(attrs = {'size': 5}))
    end = forms.TimeField(required = False, widget = forms.TextInput(attrs = {'size': 5}))

    def clean(self):
        data = super(TTItemForm, self).clean()
        start, end = data.get('start'), data.get('end')
        message = u'Введите и начало, и конец, или не вводите ни то, ни другое'
        # TODO - raise error on empty field
        if start and not end:
            raise forms.ValidationError(message)
        elif end and not start:
            raise forms.ValidationError(message)
        return data
        

TTItemFormset = inlineformset_factory(Gym, TTItem, form = TTItemForm,
                                      extra = 2, can_delete = False)

date_field = lambda label: forms.DateField(
    label = label,
    widget = forms.TextInput(attrs = {'class': 'date', 'size': 8}))

class TTFixForm(TTItemForm):
    class Meta:
        model = TTFix
        field = ['date', 'start', 'end', 'comment']

    comment = forms.CharField(required = False, widget = forms.TextInput(
        attrs = {'size': 40}))
    date = date_field(u'Дата')

TTFixFormset = inlineformset_factory(Gym, TTFix, form = TTFixForm,
                                     extra = 1, can_delete = True)
        

class PeriodForm(forms.ModelForm):
    class Meta:
        model = Period
        fields = ['gym', 'start', 'end']

    gym = forms.ModelChoiceField(queryset = Gym.objects.all(),
                                 widget = forms.HiddenInput)
    start = date_field(u'Начало')
    end = date_field(u'Конец')

    def clean(self):
        data = super(PeriodForm, self).clean()
        if data.get('start') and data.get('end') and data.get('gym'):
            # проверим, что период не пересекается ни с одним другим периодом
            gym, start, end = data['gym'], data['start'], data['end']
            qs = gym.periods.filter(Q(start__gt = start, start__lt = end) |
                                    Q(end__gt = start, end__lt = end))
            if self.instance and self.instance.id: # кроме себя самого
                qs = qs.exclude(id = self.instance.id)
            conflicts = list(qs)
            if conflicts:
                raise forms.ValidationError(
                    u'Данный период пересекается с уже сущеуствующими периодами '
                    u'действия расписания для этого зала: %s' %
                    ', '.join(map(unicode, conflicts)))
        return data
