# -*- encoding: utf-8 -*-

import datetime

from django.db import models

from utils import make_url


week_days = range(7)
weekday_names = dict(zip(
    week_days,
    u'Понедельник Вторник Среда Четверг Пятница Суббота Воскресенье'.split()))

format_time = lambda t: t.strftime('%H:%M')


class TTBase(models.Model):
    gym = models.ForeignKey('Gym')
    start = models.TimeField(null = True, blank = True)
    end = models.TimeField(null = True, blank = True)
    comment = models.TextField(blank = True, default = '')

    class Meta:
        abstract = True

    def __unicode__(self):
        if self.start and self.end:
            return u'%s - %s' % (format_time(self.start), format_time(self.end))
        return ''


class TTItem(TTBase):
    day_of_week = models.IntegerField()
    period = models.ForeignKey('Period')
    

class TTFix(TTBase):
    date = models.DateField()
    is_fix = True


class Gym(models.Model):
    title = models.CharField(max_length = 250)

    get_timetable_edit_url = make_url('timetable_edit')

    class Meta:
        verbose_name = u'Зал'
        verbose_name_plural = u'Залы'
        ordering = ['title']

    def __unicode__(self):
        return self.title


class Period(models.Model):
    start = models.DateField(u'Начало')
    end = models.DateField(u'Конец')
    gym = models.ForeignKey(Gym, related_name = 'periods')

    class Meta:
        ordering = ['start', 'end']

    def applies(self, date):
        return self.start <= date and self.end >= date 

    @classmethod
    def get_current(cls, gym):
        try:
            today = datetime.date.today()
            return cls.objects.get(gym = gym, start__lte = today, end__gte = today)
        except cls.DoesNotExist:
            pass

    def __unicode__(self):
        return u'%s - %s' % (self.start, self.end)

